# Storybook Server

[Storybook](https://storybook.js.org/) is an open source tool for building UI
components and pages in isolation. It streamlines UI development, testing, and
documentation.

This module provides an endpoint for Storybook to use with its server rendering
feature, allowing components to be rendered directly with Drupal/Twig.

## Assumptions

- Your components are stored somewhere under `themes/custom`
- You have nodejs installed
- Component ids in Storybook stories only contain alphanumeric characters, `/`,
  `_`, and  `-`
- Component templates use the file extension `.twig` (instead of `.html.twig`) -
  this prevents Drupal from picking up templates and rendering them when they
  share a name

## Setup

- Install this module
  ```
  composer require drupal/storybook_server
  ```
- In Drupal, grant permission to access Storybook Server to a privileged role at
  `/admin/people/permissions`. Log in to Drupal with that role.
- If you haven't got a `services.yml`, create one:
  ```
  cp web/sites/default/default.services.yml web/sites/default/services.yml
  ```
- In `services.yml` set `cors.config.enabled` to `true`, add the Storybook
  site url to `cors.config.allowedOrigins` (by default this is
  `http://localhost:6006` but check the URL when Storybook is started later and
  alter as necessary), and set `cors.config.supportsCredentials` to `true`
  ```
  cors.config:
    enabled: true
    # Specify allowed headers, like 'x-allowed-header'.
    allowedHeaders: []
    # Specify allowed request methods, specify ['*'] to allow all possible ones.
    allowedMethods: []
    # Configure requests allowed from specific origins.
    allowedOrigins: ['http://localhost:6006']
    # Sets the Access-Control-Expose-Headers header.
    exposedHeaders: false
    # Sets the Access-Control-Max-Age header.
    maxAge: false
    # Sets the Access-Control-Allow-Credentials header.
    supportsCredentials: true
  ```
>>>
💥 For better security, put the above settings in a separate `services.yml` file
that is not loaded in production.
>>>
- In your custom theme, create a folder for your stories. For example:
  #### **`web/themes/custom/mytheme/components/button/button.stories.json`**
  ```
  {
    "title": "Button",
    "stories": [
      {
        "name": "Button",
        "parameters": {
          "server": {
            "id": "components/button/button"
            "theme": "mytheme",
            "raw": true
          }
        },
        "argTypes": {
          "text": {
            "control": "text"
          }
        },
        "args": {
          "text":  "Button"
        }
      }
    ]
  }
  ```
  #### **`web/themes/custom/mytheme/components/button/button.twig`**
  ```
  <h1>{% trans %}Title{% endtrans %}</h1>
  <button class="widget--button">{{ text }}</button
  ```
  The `id` in `button.stories.json` is the path to the `stories.json` file under
  `themes/custom/mytheme`, with the `.stories.json` file extension removed
- Install Storybook with npx
  ```
  cd mydrupalproject
  npx sb init --type server
  ```
- Create a configuration for previews and import the configuration provided by
  this module (if `.storybook/preview.js` already exists, overwrite it)
  #### **`.storybook/preview.js`**
  ```
  `import fetchStoryHtml from '../web/modules/custom/storybook_server/fetchStoryHtml';`

  export const parameters = {
    server: {
      // Replace this with your Drupal site URL, or an environment variable.
      url: `https://[your-drupal-site-url]`,
      fetchStoryHtml,
    },
  }
  ```
- Edit `.storybook/main.js` to point to your component folder(s)
  ```
  module.exports = {
    "stories": [
      "../web/themes/custom/mytheme/components/**/*.stories.json"
    ],
    "addons": [
      "@storybook/addon-links",
      "@storybook/addon-essentials"
    ]
  }
  ```
- Start Storybook with `npm run storybook` or `yarn storybook`

## Rendering and Including CSS

Use the `raw` toggle to select whether a component renders fully encapsulated by
the selected theme, or in a raw standalone format.

When rendering in a raw format, you will probably want to include some CSS.
Doing this will depend on how your theme is structured. See
https://storybook.js.org/docs/react/configure/styling-and-css

For example, if your components have an associated CSS/Sass file that is
compiled into one CSS file for the theme, you can simply do the following in
`preview.js`:

```
import css from '../web/themes/custom/mytheme/mytheme.css';
```

This will cause that CSS to be loaded for all stories.
