<?php

namespace Drupal\storybook_server\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides an endpoint for Storybook to query.
 * @see https://github.com/storybookjs/storybook/tree/next/app/server
 */
class ServerEndpointController extends ControllerBase {

  /**
   * Render a Twig template from a Storybook component directory.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\Response|array
   * @throws \Throwable
   * @throws \Twig\Error\RuntimeError
   */
  public function render(Request $request) {
    $response = new Response();
    $component = $request->query->get('_storybook');
    $component = preg_replace('/[^[:alnum:]\/_-]/', '', $component);
    $theme = $request->query->get('_storybook_theme');
    $theme = preg_replace('/[^[:alnum:]_-]/', '', $theme);
    $raw = $request->query->getBoolean('_storybook_raw');
    $filename = "themes/custom/$theme/$component.twig";
    // Check that the filepath isn't trying to break out of themes/custom
    if (strpos(realpath(\Drupal::root() . "/$filename") , \Drupal::root() . "/themes/custom/") !== 0) {
      $response->setStatusCode(500);
    }
    else {
      // Load the Twig theme engine so we can use twig_render_template().
      include_once \Drupal::root() . '/core/themes/engines/twig/twig.engine';
      $markup = (string) twig_render_template($filename, $request->query->all());
      if ($raw) {
        $response->setStatusCode(200);
        $response->setContent($markup);
        return $response;
      }
      else {
        return [
          '#markup' => $markup,
        ];
      }
    }
  }
}
